% P47spectrum1.m Fewer photons less noise-1
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2014. All rights reserved.
 
%% init vars
post_title = 'P47spectrum1';
% modify this for your computer
dirname = ['L:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname 'code']);
% MakeBlogPostFileLists(post_title);

%% prep data
tube_voltage = 120; % kiloVolts  
[~,~,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');
tissues2 = {'soft_tissue','bone_cortical'}; 
specdat.mus = MusFromMaterials(tissues2,specdat.egys);
   % filter through typical object
thk_object = [10 .25]';  % soft tissue, cortical bone thicknesses, gm/cm^2
specdat.specnum = specdat.specnum.*exp(-specdat.mus*thk_object);

%% compute CRLB and SNR using complete spectrum
M = specdat.mus;
Cfull = diag(specdat.specnum(:)); % covariance with full spectrum
fprintf('\nFull spectrum CRLB and SNR\n');
CRLBfull = inv(M'*Cfull*M);
disp(CRLBfull);
vars_full = diag(CRLBfull);
cov_full = CRLBfull(1,end);
dA = [1; -1]; %[1; -.1];
SNR_full = dA'*M'*Cfull*M*dA; % use inv of CRLB as A-vector covariance
disp(SNR_full)

%% ideal SNR from mu's--note this assumes dA = [1; -1] 
dMus = diff(specdat.mus,1,2); % difference along rows
d2ideal = trapz(specdat.egys,specdat.specnum.*(dMus.^2));
disp(d2ideal)

%% CRLB max using no-notch energy bins
nbins = 2;
[~,Ethreshold,SNRmax,CR_no_notch] = OptimNbinsThresholdsSNR(specdat,nbins,'min_bin_width',1,'dA',dA);
fprintf('no gap variances and SNR(normalized to ideal CRLB)\n');
disp((diag(CR_no_notch)./vars_full)');
SNRopt = dA'*inv(CR_no_notch)*dA;
disp(SNRopt/SNR_full);

%% 2 bin CRLB with 'notch' taken out in center of spectrum
negys = numel(specdat.egys);
Emap = containers.Map(specdat.egys,1:negys);

%% specify notch by center energy and width
Ecenter = round(Ethreshold); % center of notch is optimal interbin energy with no notch
Ewidth = 0:2:50; % notch width
Enotch = Ecenter + [-Ewidth(:) Ewidth(:)]/2;

%% compute the variances and SNR vs. notch width
ndim = 2; % size of A-vector
nnotch = size(Enotch,1);
vars = zeros(nnotch,ndim);
snrs = zeros(nnotch,1);
covs = zeros(nnotch,1);
idxs = cell(nbins,1);
M = zeros(nbins,ndim);
lambdas = zeros(nbins,1);
for kn = 1:nnotch %630
      % the indexes to energies for each bin
    Elow = round(Enotch(kn,1));
    Ehigh = round(Enotch(kn,2));
    idxs{1} = 1:Emap(Elow);
    idxs{2} = Emap(Ehigh):negys;
       % M is effective mu for each bin
    for kspect = 1:nbins
        idx = idxs{kspect};
        sp = specdat.specnum(idx)';
        egys = specdat.egys(idx)';
        lambdas(kspect) = trapz(egys,sp);
        spnorm = sp/lambdas(kspect);
        for kdim = 1:size(specdat.mus,2)
            M(kspect,kdim) = trapz(egys,spnorm(:).*specdat.mus(idx,kdim));
        end
    end
    C = diag(lambdas); % covariance of measurements
    F = M'*C*M; % Fisher matrix
    CRnotch = inv(F); % CRLB for notch spectra
    vars(kn,:) = diag(CRnotch)';
    covs(kn) = CRnotch(1,end);
    snrs(kn) = dA'*F*dA;
end
fprintf('optimal gap variances and SNR (normalized to ideal CRLB)\n');
disp(min(vars)./vars_full');
disp(max(snrs)/SNR_full)

%% plot variances
var_normalized = bsxfun(@rdivide,vars,vars_full'); % normalized variances
cov_normalized = covs/abs(cov_full);
fh = figure;
set(fh,'color','w','position',[200 200 550 550]);
fontsiz = 16;
linewid = 1.5;
hold on 
plot(Ewidth,var_normalized(:,2),'-or','linewidth',2)
plot(Ewidth,var_normalized(:,1),'-db','linewidth',2)
plot(Ewidth,cov_normalized,'-*g','linewidth',2)
hold off
axis([0 50 -2 2])
ylabel('normalized var. or cov.','fontsize',fontsiz);
xlabel('notch width (keV)','fontsize',fontsiz);
set(gca,'fontsize',fontsiz,'linewidth',linewid)
hl = legend('\sigma^2_{A1}','\sigma^2_{A2}','cov(A_1,A_2)','location','east');

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'VARvsNotchWidthNotch';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',3,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot SNR
fh = figure;
set(fh,'color','w','position',[200 200 350 350]);
plot(Ewidth,snrs/SNR_full,'-k','linewidth',linewid)
axis([0 50 0 2])
ylabel('normalized SNR','fontsize',fontsiz);
xlabel('notch width (keV)','fontsize',fontsiz);
legend('SNR')
set(gca,'fontsize',fontsiz,'linewidth',linewid)

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'SNRvsNotchWidth';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',3,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

